var crossword = null;
var alphabets = null;

function debug( message ) {
    if ( document ) {
        var debug = document.getElementById( "debugDiv" );
        if ( debug ) {
            //debug.appendChild( document.createTextNode( message ) );
            //debug.appendChild( document.createElement( "br" ) );
            debug.innerText = message;
        }
    }
}

function tr( key ) {
    try {
        return localizableStrings[ key ];
    } catch ( e ) {
        debug( e );
        return key;
    }
}

function Word( x, y, word, definition, direction ) {
    this.x = x;
    this.y = y;
    this.word = word;
    this.definition = definition;
    if ( direction == undefined ) {
        this.horizontal = true;
    } else if ( direction == "horizontal" ) {
        this.horizontal = true;
    } else {
        this.horizontal = false;
    }
    this.toString = function() {
        return "( " + this.x + ", " + this.y + " ):" + this.word + " " + this.definition;
    }
}

function CrosswordCanvas( width, height ) {

    this.position = { x:0, y:0 };
    this.rect    = { left:0, top:0, width:320, height:320 };

    var canvas = document.getElementById( "board" );
    if ( canvas ) {
        var cstyle = document.defaultView.getComputedStyle( canvas );
        this.position.x   = parseInt( cstyle.getPropertyValue( "left" ) );
        this.position.y   = parseInt( cstyle.getPropertyValue( "top" ) );
        this.rect.width  = parseInt( cstyle.getPropertyValue( "width" ) );
        this.rect.height = parseInt( cstyle.getPropertyValue( "height" ) );
    }

    this.cursor = { x:4, y:0 };
    this.width   = width;
    this.height  = height;
    this.words   = new Array();

    this.state   = new Array();
    this.answer = new Array();

    this.horizontal = true;
    this.EMPTY = ' ';
    this.BOUNDARY = '*';

    this.isLocked = false;
    this.isCompleted = false;

    this.completeTitle = new Image();
    this.completeTitle.src = "Images/Complete.png";

    this.helpCount = 0;

//     this.completeTitle.width = 200;
//     this.completeTitle.height = 100;

    this.setRect     = Crossword__setRect;
    this.addWord     = Crossword__addWord;
    this.repaint     = Crossword__repaint;
    this.drawBoard   = Crossword__drawBoard;
    this.drawLetters = Crossword__drawLetters;
    this.drawHint    = Crossword__drawHint;
    this.drawCursor  = Crossword__drawCursor;
    this.drawTitle   = Crossword__drawTitle;
    this.setCompleteTitle = Crossword__setCompleteTitle;

    this.changeDirection = Crossword__changeDirection;
    this.changeCursorPosition = Crossword__changeCursorPosition;

    this.getWordAt       = Crossword__getWordAt;

    this.moveUp = Crossword__moveUp;
    this.moveLeft = Crossword__moveLeft;
    this.moveDown = Crossword__moveDown;
    this.moveRight = Crossword__moveRight;

    this.setLetter    = Crossword__setLetter;
    this.deleteLetter = Crossword__deleteLetter;;
    this.checkAnswer  = Crossword__checkAnswer;

    this.hasVerticalSpace = Crossword__hasVerticalSpace;
    this.hasHorizontalSpace = Crossword__hasHorizontalSpace;

    this.setDefaultCursor = Crossword__setDefaultCursor;

    this.helpLetter = Crossword__helpLetter;
    this.helpWord   = Crossword__helpWord;

    // creaete board
    for ( var i = 0; i < height; i++ ) {
        var row = new Array();
        var ans = new Array();
        for ( var j = 0; j < width; j++ ) {
            row.push( this.BOUNDARY );
            ans.push( this.EMPTY );
        }
        this.state.push( row );
        this.answer.push( ans );
    }
//     this.state[ 0 ][ 0 ] = this.EMPTY;
//     this.state[ 0 ][ 1 ] = this.EMPTY;
//     this.state[ 4 ][ 3 ] = this.EMPTY;
    return;
}

function Crossword__getWordAt( x, y ) {
//     debug( this.words.length );
    for ( var i = 0; i < this.words.length; i++ ) {
        var word = this.words[ i ];
//         debug( word );
        if ( word.horizontal == this.horizontal ) {
            if ( word.horizontal ) {
                if ( word.y == y && word.x <= x && x < word.x + word.word.length ) {
                    return word;
                }
            } else {
                if ( word.x == x && word.y <= y && y < word.y + word.word.length ) {
                    return word;
                }
            }
        }
    }
    return null;
}

function Crossword__setRect( rect ) {
//    try {
    this.rect.left = rect.left;
    this.rect.top = rect.top;
    this.rect.width = rect.width;
    this.rect.height = rect.height;
//   } catch ( e ) {
//       debug( e );
//       throw e;
//   }
}


function Crossword__addWord( word ) {
    var upper = word.word.toUpperCase();
    if ( word.horizontal ) {
        for ( var i = 0; i < word.word.length; i++ ) {
            this.state[ word.y ][ word.x + i ] = this.EMPTY;
            this.answer[ word.y ][ word.x + i ] = upper.charAt( i );
        }
    } else {
        for ( var i = 0; i < word.word.length; i++ ) {
            this.state[ word.y + i ][ word.x ] = this.EMPTY;
            this.answer[ word.y + i ][ word.x ] = upper.charAt( i );
        }
    }
    this.words.push( word );
}

function Crossword__repaint() {
    try {
        var canvas = document.getElementById( "board" );
        if ( canvas ) {
            var context = canvas.getContext( "2d" );
            if ( context ) {
                context.save();
                context.clearRect( this.rect.left, this.rect.top,
                                   this.rect.width, this.rect.height );
                this.drawBoard( context );
                this.drawHint( context );
                this.drawCursor( context );
                this.drawLetters( context );
                if ( this.isCompleted ) {
                    this.drawTitle( context );
                }
                context.restore();
            }
        }
    } catch ( e ) {
        debug( e );
    }
}

function Crossword__setCompleteTitle( src ) {
    this.completeTitle.src = src;
}

function Crossword__drawTitle( context ) {
    var x = this.rect.width / 2 - this.completeTitle.width / 2 + this.rect.left;
    var y = this.rect.height / 2 - this.completeTitle.height / 2 + this.rect.top;
    debug( "X = " + x + ", Y = " + y + ": "  );
    context.drawImage( this.completeTitle, x, y, this.completeTitle.width, this.completeTitle.height );//60, 110, 200, 100 );//x, y, this.completeTitle.width, this.completeTitle.height  );
    //context.drawImage( alphabets[ 'A' ], 10, 60, 110, 200, 10x, y, dx, dy );
}

function Crossword__drawBoard( context ) {
    var dx = parseFloat( this.rect.width ) / this.width;
    var dy = parseFloat( this.rect.height ) / this.height;
    context.strokeStyle = "black";//"#440000";
    context.lineWidth = 1.0;
    context.fillStyle = "#400";
    //context.fillRect( this.rect.left, this.rect.top, this.rect.width, this.rect.height );
    for ( var i = 0; i < this.height; i++ ) {
        var y = this.rect.top + dy * i;
        var x = this.rect.left;
        for ( var j = 0; j < this.width; j++, x += dx ) {
            if ( this.state[ i ][ j ] == this.BOUNDARY ) {
//                 context.fillStyle   = "#440000";
//                 context.fillRect( x, y, dx, dy );
            } else {
                context.fillStyle   = "white";
                if ( this.horizontal ) {
                    if ( ( j > 0 && this.state[ i ][ j - 1 ] != this.BOUNDARY ) 
                         || ( j < this.width - 1 && this.state[ i ][ j + 1 ] != this.BOUNDARY ) ) {
                        context.fillStyle   = "#fea";
                    }
                } else if ( ( i > 0 && this.state[ i - 1 ][ j ] != this.BOUNDARY )
                            || ( i < this.height - 1 && this.state[ i + 1 ][ j ] != this.BOUNDARY ) ) {
                    context.fillStyle   = "#fea";
                }

                context.fillRect( x, y, dx, dy );
                context.rect( x, y, dx, dy );
                context.stroke();
            }
        }
    }
//    context.restore();
}

function Crossword__drawLetters( context ) {
    var dx = parseFloat( this.rect.width ) / this.width;
    var dy = parseFloat( this.rect.height ) / this.height;
    context.strokeStyle = "#440000";
    for ( var i = 0; i < this.height; i++ ) {
        var y = this.rect.top + dy * i;
        var x = this.rect.left;
        for ( var j = 0; j < this.width; j++, x += dx ) {
            if ( this.state[ i ][ j ] == this.BOUNDARY 
                 || this.state[ i ][ j ] == this.EMPTY ) {
                continue;
            }
            if ( alphabets[ this.state[ i ][ j ] ] ) {
                context.drawImage( alphabets[ this.state[ i ][ j ] ], x, y, dx, dy );
            }
        }
    }
//    context.restore();
}

function Crossword__drawHint( context ) {
    var word = this.getWordAt( this.cursor.x, this.cursor.y );
    var hint = "";
    if ( word ) {
        hint = word.definition;
        var dx = parseFloat( this.rect.width ) / this.width;
        var dy = parseFloat( this.rect.height ) / this.height;
        context.strokeStyle = "#440000";
        context.lineWidth = "1.0";
        context.fillStyle = "#faa";
        for ( var i = 0; i < word.word.length; i++ ) {
            var x;
            var y;
            if ( this.horizontal ) {
                x = this.rect.left + dx * ( word.x + i );
                y = this.rect.top  + this.cursor.y * dy;
            } else {
                x = this.rect.left + this.cursor.x * dx;
                y = this.rect.top  + ( word.y + i ) * dy;
            }
            //debug( "( " + x + ", " + y + " )" );
            context.fillRect( x, y, dx, dy );
            context.rect( x, y, dx, dy );
            context.stroke();
        }
        //context.restore();
    }

    var hintDiv = document.getElementById( "hintDiv" );
    if ( hintDiv ) {
        hintDiv.innerHTML = hint;
    }
}

function Crossword__drawCursor( context ) {
    var dx = parseFloat( this.rect.width ) / this.width;
    var dy = parseFloat( this.rect.height ) / this.height;
    var x = this.rect.left + dx * this.cursor.x;// + dx / 2;
    var y = this.rect.top + dy * this.cursor.y - 1;// + dy / 2;

    context.save();
    context.strokeStyle = "rgba( 255, 0, 0, 0.75 )";
    context.shadowColor = "rgba( 0, 0, 0, 0.4 )";
    context.shadowOffsetX = "0";
    context.shadowBlur = "2";
    context.shadowOffsetY = "2";
    context.lineJoin = "round";
    context.lineWidth=2.0;
    context.rect( x, y, dx, dy );
    context.stroke();
    context.restore();

//     var rx = dx / 2 - 3;
//     var ry = dy * 2;
//     context.fillStyle = "#8a8";//strokeStyle = "#0a8";
//     context.arc( x, y, rx, ry, 0, 360 );//x + dx / 2, y + dy / 2, dx / 2, dy / 2, 0, 360);//, dy, 0, 360 );
//     context.fill();

    //context.strokeStyle = "#0a8";
    //context.lineWidth = 4.0;
    //context.rect( this.rect.left + dx * this.cursor.x + 2,
//                   this.rect.top + dy * this.cursor.y + 2,
//                   dx - 5, dy - 5 );
//     context.stroke();
//    context.restore();    
}

function Crossword__changeDirection() {
    this.horizontal = !this.horizontal;
    this.repaint();
}


function Crossword__changeCursorPosition( event ) {
    if ( crossword ) {
        var x = event.x - crossword.position.x;
        var y = event.y - crossword.position.y;
        var dx = parseFloat( this.rect.width ) / this.width;
        var dy = parseFloat( this.rect.height ) / this.height;
        var row = parseInt( y / dy );
        var col = parseInt( x / dx );
        debug( " ( " + col + ", " + row + " )" );
        if ( this.state[ row ][ col ] != this.BOUNDARY ) {
            this.cursor.x = col;
            this.cursor.y = row;
            this.repaint();
        }
    }
}


function Crossword__moveUp() {
    if ( this.cursor.y > 0 && this.state[ this.cursor.y - 1 ][ this.cursor.x ] != this.BOUNDARY ) {
        this.cursor.y --;
        this.repaint();
        return true;
    }
    return false;
}

function Crossword__moveLeft() {
    if ( this.cursor.x > 0 && this.state[ this.cursor.y ][ this.cursor.x - 1 ] != this.BOUNDARY ) {
        this.cursor.x --;
        this.repaint();
        return true;
    }
    return false;
}

function Crossword__moveDown() {
    if ( this.cursor.y < this.height - 1 && this.state[ this.cursor.y + 1 ][ this.cursor.x ] != this.BOUNDARY ) {
        this.cursor.y ++;
        this.repaint();
        return true;
    }
    return false;
}

function Crossword__moveRight() {
    if ( this.cursor.x < this.width - 1 && this.state[ this.cursor.y ][ this.cursor.x + 1 ] != this.BOUNDARY ) {
        this.cursor.x ++;
        this.repaint();
        return true;
    }
    return false;
}

function Crossword__setLetter( letter ) {
    if ( this.isLocked ) return;
    var x = this.cursor.x;
    var y = this.cursor.y;
    //debug( "#4 " + this.horizontal + ", " + this.hasHorizontalSpace( x, y ) + ", " + this.hasVerticalSpace( x, y ) );
    if ( this.state[ y ][ x ] != this.BOUNDARY ) {
        this.state[ y ][ x ] = letter;
        if ( this.horizontal ) {
            if ( this.hasHorizontalSpace( x, y ) ) {
                if ( this.moveRight() == false ) this.repaint();
            } else if ( this.hasVerticalSpace( x, y ) ) {
                this.moveDown();
                this.changeDirection();
            }
        } else {
            if ( this.hasVerticalSpace( x, y ) ) {
                if ( this.moveDown() == false ) this.repaint();
            } else {
                this.moveRight();
                this.changeDirection();
            }
        }
    }
}

function Crossword__checkAnswer() {
    var count_diff = 0;
    for ( var i = 0; i < this.height; i++ ) {
        for ( var j = 0; j < this.width; j++ ) {
            if ( this.answer[ i ][ j ] != this.EMPTY ) {
                if ( this.answer[ i ][ j ] != this.state[ i ][ j ] ) {
                    count_diff ++;
                    return false;
                }
            }
        }
    }

    //debug( "different : " + count_diff );
    if ( count_diff == 0 ) {
        this.isLocked = true;
        this.isCompleted = true;
        this.repaint();
        return true;
    }
    return false;
    //return count_diff;
}

function Crossword__hasVerticalSpace( x, y ) {
    if ( x < 0 || x >= this.width ) return false;
    if ( y < 0 || y >= this.height ) return false;
    if ( y > 0 && this.state[ y - 1 ][ x ] != this.BOUNDARY ) return true;
    if ( y < this.height - 1 && this.state[ y + 1 ][ x ] != this.BOUNDARY ) return true;
    return false;
}

function Crossword__hasHorizontalSpace( x, y ) {
    if ( x < 0 || x >= this.width ) return false;
    if ( y < 0 || y >= this.height ) return false;
    if ( x > 0 && this.state[ y ][ x - 1 ] != this.BOUNDARY ) {
        return true;
    }
    if ( x < this.width - 1 && this.state[ y ][ x + 1 ] != this.BOUNDARY ) {

        return true;
    }
    return false;
}

function Crossword__deleteLetter( letter ) {
    if ( this.isLocked ) return;
    this.state[ this.cursor.y ][ this.cursor.x ] = this.EMPTY;
    if ( this.horizontal ) {
        if ( this.moveLeft() == false ) {
            this.repaint();
        }
    } else {
        if ( this.moveUp() == false ) {
            this.repaint();
        }
    }
}

function Crossword__setDefaultCursor() {
    for ( var i = 0; i < this.height; i++ ) {
        for ( var j = 0; j < this.width; j++ ) {
            if ( this.state[ i ][ j ] != this.BOUNDARY ) {
                this.cursor = { x:j, y:i};
                return;
            }
        }
    }
}




function clickBoard( event ) {
    if ( this.isLocked ) return;
    if ( crossword ) {
        crossword.changeCursorPosition( event );
    }
//     debug( event.x + ", " + event.y );
}

function changeDirection( event ) {
    if ( crossword ) {
        crossword.changeDirection();
    }
}

function Crossword__helpLetter() {
    var x = this.cursor.x;
    var y = this.cursor.y;
    this.hintCount ++;
    this.state[ y ][ x ] = this.answer[ y ][ x ];
    this.repaint();
}

function Crossword__helpWord() {
    var x = this.cursor.x;
    var y = this.cursor.y;
    if ( this.horizontal ) {
        if ( this.hasHorizontalSpace( x, y ) ) {
            var word = this.getWordAt( x, y );
            if ( word ) {
                for ( var i = 0; i < word.word.length; i++ ) {
                    this.state[ word.y ][ word.x + i ] = this.answer[ word.y ][ word.x + i ];
                    //this.moveRight();
                    this.hintCount ++;
                }
            }
        }
    } else if ( this.hasVerticalSpace( x, y ) ) {
        var word = this.getWordAt( x, y );
        debug( "#2" + word );
        if ( word ) {
            for ( var i = 0; i < word.word.length; i++ ) {
                this.state[ word.y + i ][ word.x ] = this.answer[ word.y + i ][ word.x ];
                this.hintCount ++;
                //this.moveDown();
            }
        }
    }

    this.repaint();
}

function hintLetter( event ) {
    debug( "letter" );
    if ( crossword ) {
        crossword.helpLetter();
        crossword.checkAnswer();
    }
}

function hintWord( event ) {
    debug( "word" );
    if ( crossword ) {
        crossword.helpWord();
        crossword.checkAnswer();
    }
}

var outputBuffer = "";
var command = null;

function outputHandler( output ) {
    //debug( "output" + outputBuffer );
    outputBuffer += output;
}

function endHandler( com ) {
    debug( "END!!!!!!! "  + outputBuffer.length );
    command = null;
    eval( outputBuffer );
    crossword = new CrosswordCanvas( 16, 16 );
    for ( var i = 0; i < AnswerWords.length; i++ ) {
        crossword.addWord( AnswerWords[ i ] );
    }
    crossword.repaint();
}

function newCrossword( event ) {
    debug( "new" );
//     if ( window.widget ) {
//         crossword = null;
//         if ( command ) {
//             command.cancel();
//             command = null;
//         }
//         outputBuffer = "";
//         command = widget.system( "/usr/bin/python generate_crossword.py", endHandler );
//         command.onreadoutput = outputHandler;
//         debug( "go" );

//     }
    setTimeout( readPluginOutput, 10 );
}

//var boardData = { size:[16, 16], 
//                  words:[] };

//var userLetters = null;



//var image = new Image( 32, 32 );
//image.src = "Images/hour.pdf";

function repaintCanvas( event ) {
    if ( crossword ) {
        crossword.repaint();
    }
}


// function clearUserLetters() {
//     userLetters = new Array();
//     for ( var i = 0; i < crossword.height; i++ ) {
//         var row = new Array();
//         for ( var j = 0; j < crossword.width; j++ ) {
//             row.push( null );
//         }
//         userLetters.push( row );
//     }
// }

//var alphabets = { A:null,B:null,C:null,D:null,E:null,F:null,G:null, };
function loadAlphabets() {
    alphabets = new Array();
    var letters = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";
    for ( var i = 0; i < letters.length; i++ ) {
        var im = new Image( 16, 16 );
        var letter = letters.substring( i, i + 1 );
        im.src = "Images/" + letter + ".png";
        alphabets[ letter ] = im;
    }
//    alphabets[ "A" ].src = "Images/A.pdf";
}

function keyPressed( event ) {
    if ( crossword ) {
        if ( crossword.isLocked ) return;
        var flag_shift = event.shiftKey;
        var flag_alt = event.altKey;
        var flag_ctrl = event.ctrlKey;
        var flag_meta = event.metaKey;
        var code = event.keyCode;
//    debug( "SHIFT " + flag_shift + " CODE " + code );
        if ( flag_shift ) {
            if ( code == 63233 || code == 63232 ) {
                if ( crossword.horizontal ) {
                    crossword.changeDirection();
                }
            } else if ( code == 63235 || code == 63234 ) {
                if ( crossword.horizontal == false ) {
                    crossword.changeDirection();
                }
            }
        } else {
            if ( code == 63234 ) {
                crossword.moveLeft();
            } else if ( code == 63232 ) {
                crossword.moveUp();
            } else if ( code == 63235 ) {
                crossword.moveRight();
            } else if ( code == 63233 ) {
                crossword.moveDown();
            } else if ( code == 8 ) {
                crossword.deleteLetter();
            }
        }
        event.stopPropagation();
    }
    //event.preventDefault();
}

function keyDown(event) {
    if ( crossword ) {
        if ( crossword.isLocked ) return;
        var flag_shift = event.shiftKey;
        var flag_alt = event.altKey;
        var flag_ctrl = event.ctrlKey;
        var flag_meta = event.metaKey;
        var code = event.keyCode;
        if ( code >= 65 && code <= 90 ) {
            //debug( "down" + event.keyCode );
            crossword.setLetter( "ABCDEFGHIJKLMNOPQRSTUVWXYZ".charAt( code - 65 ) );
            crossword.checkAnswer();
        }
    }
}

var AnswerWords = new Array();
//AnswerWords.push( new Word( 0, 4, "crossword", "this game", "horizontal" ) );
//AnswerWords.push( new Word( 2, 2, "crossword", "1", "vertical" ) );
AnswerWords.push( new Word( 5, 10, "trial", "version", "horizontal" ) );

//var im = new Image( 32, 32 );
function setup( event ) {
    
    crossword = new CrosswordCanvas( 16, 16 );
    for ( var i = 0; i < AnswerWords.length; i++ ) {
        crossword.addWord( AnswerWords[ i ] );
    }
//     crossword.addWord( new Word( 0, 4, "crossword", "this game", "horizontal" ) );
//     crossword.addWord( new Word( 4, 0, "crossword", "this game you are playing", "vertical" ) );
//     crossword.addWord( new Word( 5, 14, "testversion", "text", "horizontal" ) );

    loadAlphabets();
    //clearUserLetters();

    //crossword.state[ 0 ][ 4 ] = "C";
    //userLetters[ 3 ][ 3 ] = "A";

    window.addEventListener( "keypress", keyPressed, true );
    window.addEventListener( "keydown", keyDown, true );
    //window.keydown = keyPressed;

    new ButtonListener( document.getElementById( "directionButton" ),
                        changeDirection, "Change direction" );
    new ButtonListener( document.getElementById( "hintLetterButton" ),
                        hintLetter, "Help a letter" );
    new ButtonListener( document.getElementById( "hintWordButton" ),
                        hintWord, "Help a word" );
    new ButtonListener( document.getElementById( "newButton" ),
                        newCrossword, "New Crossword" );

//     new ButtonListener( document.getElementById( "directionButton" ),
//                         changeDirection, "CHANGE_DIRECTION" );
//     new ButtonListener( document.getElementById( "hintLetterButton" ),
//                         hintLetter, "SHOW_LETTER" );
//     new ButtonListener( document.getElementById( "hintWordButton" ),
//                         hintWord, "SHOW_WORD" );
//     new ButtonListener( document.getElementById( "newButton" ),
//                         newCrossword, "NEW_CROSSWORD" );


    crossword.setDefaultCursor();
    repaintCanvas();
//    setTimeout( repaintCanvas, 1000 );
//    debug( im );
//     debug( "setup" );

    if ( CrosswordPlugin ) {
        debug( "GO" );
        setTimeout( readPluginOutput, 10 );
    } else {
        debug( "NO plugin" );
    }

}

function readPluginOutput() {
    var output = CrosswordPlugin.getCrosswordAnswers( 0 );
    var lines = output.split( "\n" );
    //var nextAnswer = new Array();
    var nextCanvas = new CrosswordCanvas( 16, 16 );
    var reg_line = new RegExp( "(H|V):(\\d+),(\\d+):([a-zA-Z]+):(.*)" );
    var count_words = 0;
    for ( var i = 0; i < lines.length; i++ ) {
        var items = reg_line.exec( lines[ i ] );
        if ( items ) {
            var dir = "horizontal";
            if ( items[ 1 ] == "H" ) {
                dir = "horizontal";
            } else if ( items[ 1 ] == "V" ) {
                dir = "vertical";
            } else {
                continue;
            }
            var x = parseInt( items[ 2 ] );
            var y = parseInt( items[ 3 ] );
            var label = items[ 4 ];
            var def = items[ 5 ];
            var dictitem = new Word( x, y, label, def, dir );
            //nextAnswer.psh( dictitem );
            nextCanvas.addWord( dictitem );
            count_words ++;
        }
    }
    if ( count_words > 4 ) {
        nextCanvas.setDefaultCursor();
        //debug( output );
        crossword.isLocked = true;
        crossword = null;
        crossword = nextCanvas;
        
        nextCanvas.repaint();
    }
}


// buttons

function ButtonListener( element, target, popup ) {
    this.element = element;
    this.target = target;
    this.enabled   = true;

    this.mouseDown = _ButtonListener__mouseDown;
    this.mouseUp   = _ButtonListener__mouseUp;

    this.enable    = _ButtonListener__enable;


    var self = this;
    element.addEventListener( 'mousedown', function() { self.mouseDown(); } );
    element.addEventListener( 'mouseup', function() { self.mouseUp(); } );


    this.showPopUp = _ButtonListener__showPopUp;
    this.hidePopUp = _ButtonListener__hidePopUp;
    if ( popup ) {
        var elem = document.createElement( "div" );
        var bstyle = document.defaultView.getComputedStyle( element );
        var x = parseInt( bstyle.getPropertyValue( "left" ) );
        var y = parseInt( bstyle.getPropertyValue( "top" ) );
        elem.style.position = "absolute";
        elem.style.left = ( x + 24 ) + "px";
        elem.style.top = ( y + 20 ) + "px";
//         elem.style.width = "80px";
//         elem.style.height = "20px";
        elem.style.background = "#bef";
        elem.style.color="black";
        elem.style.fontFamily="Lucida Grande";
        elem.style.fontSize="10px";
        elem.style.padding = "2px;";
        elem.style.border = "#4c8 1px solid";
        elem.style.display = "none";
        elem.style.opacity = "0.7";
        elem.style.zIndex = "999";
        elem.appendChild( document.createTextNode( tr( popup ) ) );
        
        document.firstChild.appendChild( elem );
        this.popup = elem;
        element.addEventListener( 'mouseover', function() { self.showPopUp(); } );
        element.addEventListener( 'mouseout', function() { self.hidePopUp(); } );
    } else {
        this.popup = null;
    }
    
}

function _ButtonListener__enable( flag ) {
    if ( flag == this.enabled ) return;
    this.enabled = flag;
    if ( flag ) {
        this.element.style.opacity = 1.0;
    } else {
        this.element.style.opacity = 0.5;
    }
}

function _ButtonListener__mouseDown( event ) {
    this.hidePopUp();
    if ( this.enabled == false ) return;
    this.element.style.background = "url(Images/Button2.png)";
}

function _ButtonListener__mouseUp( event ) {
    this.hidePopUp();
    if ( this.enabled == false ) return;
    this.element.style.background = "url(Images/Button1.png)";
    this.target( event );
}

function _ButtonListener__showPopUp( event ) {
    if ( this.popup ) {
        this.popup.style.display = "block";
    }
}

function _ButtonListener__hidePopUp( event ) {
    if ( this.popup ) {
        this.popup.style.display = "none";
    }
}

function setButtonAction( element, targetFunc ) {
    var listener = new ButtonListener( element, targetFunc );
    return listener;
}
